/**
 *   @file  main.c
 *
 *   @brief   
 *      Test Code to test the Fault Management library
 *
 *  \par
 *  NOTE:
 *      (C) Copyright 2012 Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include <stdio.h>
#include <stdint.h>

/* BIOS/XDC Include Files. */
#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/SysMin.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

/* CSL Include Files */
#include <ti/csl/csl_chip.h>
#include <ti/csl/csl_cacheAux.h>
#include <ti/csl/csl_chipAux.h>

/* Fault Management Include File */
#include <ti/instrumentation/fault_mgmt/fault_mgmt.h>

/**********************************************************************
 ************************* Unit Test Functions ************************
 **********************************************************************/

Void myExceptionHook(Void)
{
    uint32_t   i, my_core_id, efr_val;
    /* Copy register status into fault management data region for Host */
    Fault_Mgmt_getLastRegStatus();
	
	my_core_id = CSL_chipReadDNUM();

	efr_val    = CSL_chipReadEFR();
	if (efr_val & 0x80000000)
	{
		/* Exception is triggered to be assumed by other core for NMI exceptions
		 * so, don't need to notify other cores since the parent core where the
		 * exception has triggered by an event would notify them, thus eliminating
		 * the recursive exceptions */

		/* Notify Host of crash and return */
	    Fault_Mgmt_notify();
	    return;
	}

    for (i = 0; i < FAULT_MGMT_NOTIFY_NUM_CORES; i++)
	{
       /* Skip the notification if it is same core */
	   if (i == my_core_id)
	       continue;
		   
	   /* Notify remote DSP cores for exception - WARNING: This will generate NMI
           pulse to the remote DSP cores */
       Fault_Mgmt_notify_remote_core(i);
	}	
    /* Notify Host of crash */
    Fault_Mgmt_notify();
}

/* Exception inducing function */
void exceptionFunc (void)
{
	System_printf ("Causing exception\n");

	/* Attempt to use B0 twice in the same execution stage.  This will
	 * cause a resource sharing exception to occur */
    asm(" \n \
		mvk 1, a1; \n \
		mvk 1, a0; \n \
	  [a1]	mvk 0, b0; \n \
	||[a0]	mv a0, b0; \n \
    ");

    /* The crash dump should show NRP as corresponding to this printf.
     * NRP is used as the PC in the crash dump since that is the return
     * address from the non-maskable interrupt generated by the
     * exception */
    System_printf ("Print after causing the exception\n");
}

/* Second function in test call stack. */
void testFunc1 (void)
{
	System_printf ("In testFunc1\n");

	/* Call the function which will cause the exception */
	exceptionFunc();
}

/* Task which causes DSP exception.  Need to create a call stack first for
 * crash dump analysis testing purposes. */
void testTask ()
{
	uint32_t coreId;
	System_printf ("In testTask\n");
	coreId = CSL_chipReadDNUM();

	if (coreId == 0) {

  	  /* Call first function in call stack */
	  testFunc1();
	}
	else
	{
		/* No operations for other cores */
		while(1);
	}
}

int32_t main(int32_t argc, char* argv[])
{
    Task_Params         taskParams;

    System_printf ("**********************************************\n");
    System_printf ("********* Fault Management Unit Test *********\n");
    System_printf ("**********************************************\n");

    /* Enable the caches. */
	CACHE_setL1DSize (CACHE_L1_32KCACHE);
	CACHE_setL1PSize (CACHE_L1_32KCACHE);

    /* Initialize the Task Parameters. */
    Task_Params_init(&taskParams);

    /* Create a task that causes an exception */
    Task_create((Task_FuncPtr)testTask, &taskParams, NULL);

    BIOS_start();
    return 0;
}

